#! /usr/local/bin/perl5

# Package drspeech
#
# $Header: /u/drspeech/src/babylon/RCS/drspeech.pm,v 1.15 1998/09/23 23:06:20 dpwe Exp $
#
# The "drspeech" PERL package.
# Lots of useful functions and variables for writing PERL scripts.

package drspeech;

=head1 NAME

drspeech - Functions and variables for writing scripts

=head1 SYNOPSIS

C<use drspeech;>

=head1 DESCRIPTION

This package sets a bunch of variable defaults, and contains signal handling,
housekeeping, and log file maintenance routines.

=head1 DEFINED VARIABLES

The following variables are defined when the C<drspeech> package is first used.
All variables are defined in package C<drspeech> and should be
qualified with C<drspeech::> when used from another package.

=over 4

=cut

### Work out the program name.

=item C<$progname>

The name of the program, defaults to C<$0>

=cut

$progname = $0;


### Set the value of $tmpdir.

=item C<$tmpdir>

The name of the directory where temporary files go; defaults to the
environment variable C<DRSPEECH_TMPDIR>, or if that is not defined,
the environment variable C<TMPDIR>, or otherwise just C<"/tmp">.

=cut

if (defined($ENV{'DRSPEECH_TMPDIR'}))
{
    $tmpdir = $ENV{'DRSPEECH_TMPDIR'};
}
elsif (defined($ENV{'TMPDIR'}))
{
    $tmpdir = $ENV{'TMPDIR'};
}
else
{
    $tmpdir = "/tmp";
}

=item C<$home>

The home directory for DRSPEECH, where all non-standard unix
executables and scripts can be found.  Defaults to the environment
variable C<DRSPEECH_HOME>, or C<"/u/drspeech"> if that variable is
undefined.

=cut

### Set the value of $home.

if (defined($ENV{'DRSPEECH_HOME'}))
{
    $home = $ENV{'DRSPEECH_HOME'};
}
else
{
    $home = "/u/drspeech";
}

=item C<$arch>

The architecture of the host.  Its value will be one
of "sun4", "sun4-sunos5", "iris", "iris4-irix5", "i586-linux", "ppc-linux".  
The value is set
from the environment variable C<DRSPEECH_ARCH>, or by evaluating the
unix C<"uname"> command if that is not set.

=cut

### Set the value of $arch.

if ($ENV{'DRSPEECH_ARCH'})
{
    $arch = $ENV{'DRSPEECH_ARCH'};
}
else
{
    $mach = `uname -m`;
    chop $mach;
    if ($?)
    {
	die "package 'drspeech' failed to execute 'uname -m' command.\n";
    }
    $os = `uname -r`;
    chop $os;
    if ($?)
    {
	die "package 'drspeech' failed to execute 'uname -r' command.\n";
    }
    $arch = 'unknown';
    $_ = $mach . ":" . $os;
    if (/^sun4.*:5\./) { $arch = 'sun4-sunos5'; }
    if (/^sun4.*:4\./) { $arch = 'sun4'; }
    if (/^IP.*:4\./) { $arch = 'iris'; }
    if (/^IP.*:5\./) { $arch = 'iris-irix5'; }
    if (/^i.86.*:2\./) { $arch = 'i586-linux'; }
    if (/^ppc:2\./) { $arch = 'ppc-linux'; }
    if ($arch eq "unknown")
    {
	die "package 'drspeech', unknown architecture.\n";
    }
}

### Set the PATH environment variable appropriately.

=item C<$ENV{'PATH'}>

The C<PATH> environement variable is set to be:

    /bin:/usr/bin:/usr/ucb/bin:${DRSPEECH_HOME}/<arch>/bin:\
          ${DRSPEECH_HOME}/share/bin

All executables should be findable from this path.

=cut
    
$ENV{'PATH'} = "/bin:/usr/bin:/usr/ucb/bin:" .
    "$home/$arch/bin:$home/share/bin";

### Set logging defaults.

=item C<$verbose>

This variable sets the level of diagnostic output.   The
default is 1 which produces reasonable levels (a few lines to a few
pages) of output for tracking the progress of the job.  A value of 0
means produce no diagnostic output.  With C<$verbose> set to 1, the
values of all command line options are logged to the diagnostic log
stream at the start of program execution.

=cut

$verbose = 1;

### Set tidying defaults.

=item C<$tidyup>

This variable controls whether certain tidyup functions are called at
exit, whether abnormal or regular.  When non-zero, all drspeech
temporary files (C<$tmpdir/drspeech_*.$$>) are deleted
at exit.

=cut

$tidyup = 1;

=head1 FUNCTIONS

=over 4

=cut

### Wrapper around exit to tidy up

=item C<&exit($code)>

You should try to call C<&drspeech::exit()> rather than C<exit()> from
your programs;  this will invoke the C<&tidyup> function to clean up
and close log files.

=cut

sub exit {
    my $exitcode=shift;
    &drspeech::tidyup();
    exit($exitcode);
}

### Housekeeping done on exit

=item C<&tidyup()>

Does housekeeping, such as removing all temporary files and closing
log files.

=cut

sub tidyup {
    unlink <$drspeech::tmpdir/drspeech_*.$$> if ($drspeech::tidyup);
    &drspeech::close_log();
}

### These are replacements for warn and die

=item C<&warn($string)>

Will print error messages to C<STDERR> and also the log stream if
it is not the same file descriptor. By default, the handler for
perl C<warn()> is set to be C<&drspeech::warn()>, see below for
details.

=cut

sub warn {
    my($fd1,$fd2);
    my $error = shift;
    my $msg = "WARNING - " . $error;

    if ($log) {
	$fd1=fileno(STDERR);
	$fd2=fileno($log);

	if ($fd1!=$fd2) {
	    print $log $msg;
	}
    }
    print STDERR "$progname: $msg";
}

=item C<&die($string)>

Will print error messages to C<STDERR> and also the log stream if
it is not the same file descriptor.  By default, the handler for
perl C<die()> is set to be C<&drspeech::die()>, see below for
details.  Will exit with an error after calling
C<&tidyup>.

=cut

sub die {
    my($fd1,$fd2);
    my $error = shift;
    
    my $msg = "ERROR - " . $error;

    if ($log) {
	$fd1=fileno(STDERR);
	$fd2=fileno($log);

	if ($fd1!=$fd2) {
	    print $log $msg;
	}
    }
    &drspeech::tidyup();
    print STDERR "${progname}: ${msg}";
    exit 1;
}

### Extra function inform, which is "wimpy warn"... simply inform the user
### of what's going on

=item C<&inform($string)>

Will print information messages to the log stream (if it exists) and also
C<STDERR> and if the log stream is not C<STDERR> or C<STDOUT>.

=cut

sub inform {
    my($fd1,$fd2,$fd3);
    my $msg = join("",@_);
    my $msg2=$msg;
    my $lastchar=chop($msg2);

    # don't put an extra newline on if one exists already
    $lastchar=($lastchar eq "\n")?"":"\n";
    
    if ($log) {
	$fd1=fileno(STDERR);
	$fd2=fileno($log);
	$fd3=fileno(STDOUT);

	print $log $msg;
	if ($fd1!=$fd2 && $fd3!=$fd2) {
	    print STDERR "$progname: $msg$lastchar";
	}
    } else {
	print STDERR "$progname: $msg$lastchar";
    }
}


### Handler to catch signals

=item C<&handler()>

A generic handler function for catching signals, which prints a
message to C<$log> and STDERR, calls C<&tidyup>, and exits
with an error code of 1.

=cut

sub handler {
    my($fd1,$fd2);
    local($sig)=$_[0];

    print STDERR "$progname: ERROR - caught a SIG$sig, exiting.\n";

    if ($log) {
	$fd1=fileno(STDERR);
	$fd2=fileno($log);

	if ($fd1!=$fd2) {
	    print $log "ERROR - caught a SIG$sig, exiting.\n";
	}
    }
    
    &drspeech::tidyup();
    exit(1);
}

### Log files

=item C<&open_log()>

A routine to open the log file defined in C<$logname>, or C<"-"> if
that does not exist.  This is called by C<&drspeech_args::parse()> if
you requested logging by defining C<$logname> in the argument
structure, see L<drspeech_args(1)> for more detail.  Note that you can
do fancy perl-like things, such as making the log stream a pipe or
another file descriptor; see L<perlfunc/open> for more details.
The filehandle reference is placed in C<$log>.

=cut

sub open_log {
    if (!defined($logname)) {
	$logname="-";
    }

    if ($logname =~ /^[\+\|>]/) {
	open(FILE,"$logname") || die ("$progname(drspeech::open_log): Can't open \"$logname\" for writing");
    } else {
	open(FILE,">$logname") || die ("$progname(drspeech::open_log): Can't open \"$logname\" for writing");
    }

    $log=\*FILE;
}

=item C<&close_log()>

Closes the filehandle in C<$log>;  called upon exit by C<&tidyup>.

=cut
sub close_log {
    if (defined($log)) {
	close($log);
	undef($log);
    }
}

### Set up the signal catching

=back

=head1 SIGNALS

The following signals are caught automatically by perl after 
C<use drspeech>.  The C<__WARN__> and C<__DIE__> signals are
perl-internal signals which happen when C<warn()> or C<die()> is
called.

  $SIG{__WARN__}="drspeech::warn";
  $SIG{__DIE__}="drspeech::die";
  $SIG{'HUP'}="drspeech::handler";
  $SIG{'INT'}="drspeech::handler";
  $SIG{'QUIT'}="drspeech::handler";
  $SIG{'ILL'}="drspeech::handler";
  $SIG{'TRAP'}="drspeech::handler";
  $SIG{'IOT'}="drspeech::handler";
  $SIG{'ABRT'}="drspeech::handler";
  $SIG{'EMT'}="drspeech::handler";
  $SIG{'FPE'}="drspeech::handler";
  $SIG{'BUS'}="drspeech::handler";
  $SIG{'SEGV'}="drspeech::handler";
  $SIG{'SYS'}="drspeech::handler";
  $SIG{'TERM'}="drspeech::handler";
  $SIG{'XCPU'}="drspeech::handler";
  $SIG{'XFSZ'}="drspeech::handler";
  $SIG{'USR1'}="drspeech::handler";
  $SIG{'USR2'}="drspeech::handler";

=cut

$SIG{__WARN__}="drspeech::warn";
$SIG{__DIE__}="drspeech::die";
$SIG{'HUP'}="drspeech::handler";
$SIG{'INT'}="drspeech::handler";
$SIG{'QUIT'}="drspeech::handler";
$SIG{'ILL'}="drspeech::handler";
$SIG{'TRAP'}="drspeech::handler";
$SIG{'IOT'}="drspeech::handler";
$SIG{'ABRT'}="drspeech::handler";
$SIG{'EMT'}="drspeech::handler";
$SIG{'FPE'}="drspeech::handler";
$SIG{'BUS'}="drspeech::handler";
$SIG{'SEGV'}="drspeech::handler";
$SIG{'SYS'}="drspeech::handler";
$SIG{'TERM'}="drspeech::handler";
$SIG{'XCPU'}="drspeech::handler";
$SIG{'XFSZ'}="drspeech::handler";
$SIG{'USR1'}="drspeech::handler";
$SIG{'USR2'}="drspeech::handler";

=back

=head1 SEE ALSO

L<drspeech_args(1)>

=head1 AUTHORS

 David Johnson, davidj@icsi.berkeley.edu
 Eric Fosler, fosler@icsi.berkeley.edu

=cut

    
package main;

1;
