#!/usr/local/bin/perl5

# Package drspeech_parallel: Module for parallelization
#
# $Header: /u/drspeech/src/babylon/RCS/drspeech_parallel.pm,v 1.4 1997/08/05 18:36:35 bedk Exp $
# 
# $Log: drspeech_parallel.pm,v $
# Revision 1.4  1997/08/05 18:36:35  bedk
# Deleted debugging print statement
#
# Revision 1.3  1997/07/09 17:57:03  bedk
# Added pmake_only_local variable -- if true, jobs are executed only on the local
# machine.
#
# Revision 1.2  1997/07/08 16:33:48  bedk
# Added pod documentation, a flag to run some jobs locally, a logfile for the
# pmake run, and a logfile for export runs.  Also, the temporary pmake file is
# now written in $drspeech::tmpdir, not in /tmp
#
# Revision 1.1  1996/09/05 20:04:40  fosler
# Initial revision
#
#

package drspeech_parallel;

=head1 NAME

drspeech_parallel - a parallelization package

=head1 SYNOPSIS

  use drspeech;
  use drspeech_args;
  use drspeech_parallel;

=head1 DESCRIPTION

This package provides a PERL interface to the pmake and export programs,
enabling the parallelization of programs across workstations.

=head1 DEFINED VARIABLES

The following variables are defined by C<drspeech_parallel> when the
package is used:

=over 4

=item C<$drspeech_parallel::method>

A string that defines the parallelization method to use.  It defaults to
"pmake", which is currently the only valid value.

=item C<$drspeech_parallel::pmake_attributes>

A string that gives the attributes of workstations to which jobs may
be exported by pmake.  The default value is "sun4m OR sun4u".

=item C<$drspeech_parallel::pmake_numjobs>

The number of parallel jobs allowed.  The default is 2.

=item C<$drspeech_parallel::pmake_exec_locally>

If true, execute jobs locally as well as on other machines.  Default is false.

=item C<$drspeech_parallel::pmake_only_local>

If true, execute jobs only on the local machine.  Default is false.

=item C<$drspeech_parallel::pmake_jobnames_prefix>

A string that gives the base name for pmake jobs.  The default is
"job_".

=item C<$drspeech_parallel::pmake_logfile>

A string that gives the log file to which pmake output will be
written.  The default is "/dev/null".

=item C<$drspeech_parallel::pmake_prog>

A string that gives the path to the pmake program.  The default is
"pmake".

=item C<$drspeech_parallel::export_attributes>

A string that gives the attributes of workstations to which jobs may
be exported by export.  The default value is "sun4m OR sun4u".

=item C<$drspeech_parallel::export_logfile>

A string that gives the log file to which export output will be
written.  The default is "/dev/null".

=item C<$drspeech_parallel::export_prog>

A string that gives the path to the export program.  The default is
"/usr/local/bin/export".

=back

=cut

$method="pmake" if (!defined($method));

$pmake_attributes="sun4m OR sun4u" if (!defined($pmake_attributes));
$pmake_numjobs=2 if (!defined($pmake_numjobs));
$pmake_exec_locally=0 if (!defined($pmake_exec_locally));
$pmake_only_local=0 if (!defined($pmake_only_local));
$pmake_jobnames_prefix="job_" if (!defined($pmake_jobnames_prefix));
$pmake_logfile="/dev/null" if (!defined($pmake_logfile));
$pmake_prog="pmake" if (!defined($pmake_prog));

$export_attributes="sun4m OR sun4u" if (!defined($export_attributes));
$export_logfile="/dev/null" if (!defined($export_logfile));
$export_prog="/usr/local/bin/export" if (!defined($export_prog));

=head1 FUNCTIONS

=over 4

=item C<&drspeech_parallel::parallelize(@commands)>

Executes the commands in the list @commands in parallel using pmake.
The return value is the return value from pmake.

=cut

sub parallelize {
    local(@jobs)=@_;

    # switch here on different parallelization techniques

    #default
    return &parallelize_pmake($pmake_attributes,$pmake_numjobs,$pmake_exec_locally,$pmake_only_local,$pmake_jobnames_prefix,@jobs);
}

# parallelize(attributes,numjobs,prefix,command1,command2...)
# where
# attibutes are a list of export specifications (ala customs)
# numjobs is the number of jobs to do in parallel
# prefix is the prefix to use in creating job names
# command1... is a list of commands to run.


sub parallelize_pmake {
    local($attributes)=shift(@_);
    local($njobs)=shift(@_);
    local($exec_locally)=shift(@_);
    local($only_local)=shift(@_);
    local($prefix)=shift(@_);
    local(@commands)=@_;
    local($makefile)=$drspeech::tmpdir."/drspeech_pmakefile.$$";
    local($jobnum,$alljobs,$job);
    local($savepmake);
    local($command);
    local($retval);

    die("$0: parallel::parallelize_pmake: called with njobs <= 0") 
	if ($njobs<=0);

    open(MKFILE,">$makefile") || die ("$0: parallel.pl: parallelize: can't open temp file $mkfile for writing");
    print MKFILE ".EXPORT:\n";
    print MKFILE ".EXPORT: $attributes\n";

    for ($jobnum=0;$jobnum<=$#commands;$jobnum++) {
	$job=$prefix."_".$jobnum;
	$alljobs.=" ".$job;
	print MKFILE $job,": ; ",$commands[$jobnum],"\n";
    }
    
    print MKFILE ".MAIN: $alljobs\n";

    close(MKFILE);

    if (defined($ENV{"PMAKE"})) {
	$savepmake=$ENV{"PMAKE"};
    }
    $ENV{"PMAKE"}="";

    $command="$pmake_prog -l ";
    if (&drspeech_args::convert_string_to_bool($exec_locally))
    {
        $command = $command." -L -1 ";
    }
    else
    {
        $command = $command." -L 0 ";
    }
    if (&drspeech_args::convert_string_to_bool($only_local))
    {
        $command = $command." -X ";
    }
    $command = $command." -J $njobs -f $makefile 1>$pmake_logfile 2>&1";

    $retval=system("$command");

    unlink($makefile);
    if (defined($savepmake)) {
	$ENV{"PMAKE"}=$savepmake;
    } else {
	undef($ENV{"PMAKE"});
    }
    return $retval;
}

=item C<&drspeech_parallel::export($command)>

Executes $command remotely using export.  The return value is the
return value from export.

=cut

sub export {
    local($command)=shift(@_);

    $attributes =~ s/^\s*//;
    $attributes =~ s/\s+/ -attr /g;
    $attributes =~ s/^/-attr /;
    
    system("$export_prog $attributes sh -c \"$command\" 1>$pmake_log 2>&1");
    return $?;
}

=head1 AUTHORS

original package by Eric Fosler, fosler@icsi.berkeley.edu
modifications by Brian Kingsbury, bedk@icsi.berkeley.edu

=head1 SEE ALSO

drspeech(1), pmake(1), export(1)

=cut

1;
